xplor.requireVersion("2.33")


import protocol

# Total number of structures.
numberOfStructures=5000

# Base name for output pdb files.
outfilename = 'SCRIPT_STRUCTURE.pdb'

# Initialize random seed.
protocol.initRandomSeed()

# Initialize topology and parameters specific to EEFx
#
import eefxPotTools
eefxPotTools.initEEFx()

command = xplor.command
seq='''ACE CYS HIS TRP GLU SER ARG GLY SER ARG LEU VAL CYS'''

import psfGen
psfGen.seqToPSF(seq,segName='A')
psfGen.seqToPSF(seq,segName='B')

import psfGen
psfGen.addDisulfideBond("segid A and resid 13", "segid B and resid 2", bond=True)
psfGen.addDisulfideBond("segid B and resid 13", "segid A and resid 2", bond=True)


# Generate extended structure.
protocol.genExtendedStructure()

#list to hold potential terms in structure calculation
from potList import PotList
potList = PotList()
crossTerms = PotList('cross')

#compute RMSD to reference structure
#  backbone and heavy atom RMSDs will be printed in the output
#  structure files
#
#from posDiffPotTools import create_PosDiffPot
#refRMSD = create_PosDiffPot("refRMSD",
#                            "(name CA or name C or name N) and resid 1:230",
#                            pdbFile='1EZA.pdb',
#                            cmpSel="not name H*")


# lists to hold force constant settings for various stages of
# structure calculation
from simulationTools import MultRamp, StaticRamp
hiTempParams = []
# Settings for annealing stage.
rampedParams = []

# Set up NOE potential.
from noePotTools import create_NOEPot
noe = create_NOEPot('noe', 'noe.tbl')
potList.append(noe)
rampedParams.append(MultRamp(2.0,30.0, "noe.setScale( VALUE )"))

# Set up dihedral angle restraints
torsionFile='dihed.tbl'
protocol.initDihedrals(torsionFile)
from xplorPot import XplorPot
potList.append(XplorPot('CDIH'))
#hiTempParams.append( StaticRamp("potList['CDIH'].setScale(10)") )
rampedParams.append( StaticRamp("potList['CDIH'].setScale(200)") )

# Statistical torsion angle potential
#
from torsionDBPotTools import create_TorsionDBPot
torsionDBPot = create_TorsionDBPot('tDB')
potList.append( torsionDBPot )
rampedParams.append( MultRamp(.002,2,"torsionDBPot.setScale(VALUE)") )


#covalent terms
from xplorPot import XplorPot
for term in ('BOND', 'ANGL', 'IMPR'):
    potList.append( XplorPot(term) )
    pass

# Set threshold for terms in potList to allow violation analysis.
potList['ANGL'].setThreshold(5.0)   # default is 2.0
potList['IMPR'].setThreshold(5.0)   # default is 2.0
# Use default values for the rest (bond: 0.05, cdih: 5.0, noe: 0.5).

rampedParams.append(MultRamp(0.4, 1.0, "potList['ANGL'].setScale(VALUE)"))
rampedParams.append(MultRamp(0.1, 1.0, "potList['IMPR'].setScale(VALUE)"))

#nonbonded term - EEFx
from eefxPotTools import create_EEFxPot, param_LK
eefxpot=create_EEFxPot("eefxpot")
eefxpot.setVerbose(False)
potList.append(eefxpot)
rampedParams.append(MultRamp(0.1,1.0,"eefxpot.setScale(VALUE)"))
rampedParams.append(StaticRamp("potList['VDW'].setScale(0)"))

#use repel at high temp
potList.append( XplorPot("VDW") )
hiTempParams.append(StaticRamp("""protocol.initNBond(cutnb=100,
                                                     rcon=0.04,
                                                     tolerance=45,
                                                     nbxmod=5,
                                                     repel=1.2,
						     onlyCA=True)"""))  
hiTempParams.append(StaticRamp("potList['VDW'].setScale(1)"))
hiTempParams.append(StaticRamp("eefxpot.setScale(0)"))     

# Set up IVM object(s).

# IVM object for torsion-angle dynamics.
from ivm import IVM
dyn = IVM()
protocol.torsionTopology(dyn)

# IVM object for final Cartesian minimization.
minc = IVM()
protocol.cartesianTopology(minc)



temp_ini = 3500.0
temp_fin = 25.0


# Give atoms uniform weights, except for the anisotropy axis.
protocol.massSetup()


def calcOneStructure(loopInfo):
    """Calculate a structure.

    """
    # Generate an initial structure by randomizing torsion angles.
    from monteCarlo import randomizeTorsions
    randomizeTorsions(dyn)
    protocol.fixupCovalentGeom(maxIters=100, useVDW=1)

    #
    # High Temperature Dynamics Stage.
    #
    # Set torsion angles from restraints.
    # (They start satisfied, allowing the shortening of high temp dynamics.)
    from torsionTools import setTorsionsFromTable
    setTorsionsFromTable(torsionFile)

    # Initialize parameters for high temp dynamics.
    from simulationTools import InitialParams
    InitialParams(rampedParams) # overides some in rampedParams
    InitialParams(hiTempParams) # overides some in rampedParams

    #
    # Torsion angle minimization.
    #
    protocol.initMinimize(dyn,
                          potList=[term for term in potList
                                   if term.instanceName()!="eefxpot"],
                          numSteps=50,
                          printInterval=10)
    dyn.run()


    protocol.initDynamics(dyn,
                          potList=[term for term in potList
                                   if term.instanceName()!="eefxpot"],
                          bathTemp=temp_ini,
                          initVelocities=1,
                          finalTime=400,  # run for finalTime or 
                                          # numSteps * 0.001, whichever is less
                          numSteps=4000,
                          printInterval=100)


    dyn.setETolerance(temp_ini/100) # used to det. stepsize. default: temp/1000
    dyn.run()

    protocol.initNBond()

    #dynamics with all-atom repel potential
    protocol.initDynamics(dyn,
                          potList=[term for term in potList
                                   if term.instanceName()!="eefxpot"],
                          bathTemp=temp_ini,
                          finalTime=40,  # run for finalTime or 
                                         # numSteps * 0.001, whichever is less
                          numSteps=800,
                          printInterval=10)


    dyn.setETolerance(temp_ini/100) # used to det. stepsize. default: temp/1000
    dyn.run()

    InitialParams(rampedParams)

    #minmization with EEFx potential
    protocol.initMinimize(dyn,
                          potList=potList,
                          numSteps=200,
                          printInterval=10)
    dyn.run()

    #dynamics with EEFx potential
    protocol.initDynamics(dyn,
                          potList=potList,
                          bathTemp=temp_ini,
                          initVelocities=1,
                          finalTime=30,  # run for finalTime or 
                                         # numSteps * 0.001, whichever is less
                          numSteps=30000, 
                          printInterval=100)

    dyn.setETolerance(temp_ini/100) # used to det. stepsize. default: temp/1000
    dyn.run()

                          
    #
    # Simulated Annealing Stage.
    #
    # Initialize parameters for annealing.
    InitialParams(rampedParams)

    # Set up IVM object for annealing.
    protocol.initDynamics(dyn,
                          potList=potList,
                          finalTime=0.4,  # run for finalTime or 
                                          # numSteps * 0.001, whichever is less
                          numSteps=201,   
                          printInterval=100)    

    # Set up cooling loop and run.
    from simulationTools import AnnealIVM
    AnnealIVM(initTemp=temp_ini,
              finalTemp=temp_fin,
              tempStep=12.5,
              ivm=dyn,
              rampedParams=rampedParams).run()

    #
    # Torsion angle minimization.
    #
    protocol.initMinimize(dyn,
                          potList=potList,
                          printInterval=50)
    dyn.run()

    #
    # Final Cartesian minimization.
    #
    protocol.initMinimize(minc,
                          potList=potList,
                          dEPred=10)
    minc.run()


    # Write structure.



from simulationTools import FinalParams
from simulationTools import StructureLoop
StructureLoop(numStructures=numberOfStructures,
              doWriteStructures=True,
              averagePotList=potList,
              averageContext=FinalParams(rampedParams),
              pdbTemplate=outfilename,
              structLoopAction=calcOneStructure,
              genViolationStats=True,
              averageTopFraction=0.20, #report only on best 50% of structs
              averageFilename="SCRIPT_ave.pdb",   #generate regularized ave structure
              ).run()
                              
